package net.sf.sql2java.common.beans;

import javax.sql.DataSource;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.GenericObjectPool;

public class DatabaseConnection {
    private String driverClass;
    private String url;
    private String username;
    private String password;
    private String catalog;
    
    private DataSource dataSource;
    
 // -- Database Actions --------------------------------------------------------

    public void connect() throws Exception {
        // -- load the driver
        Class.forName(driverClass);

        // -- create the pool
        ObjectPool connectionPool = new GenericObjectPool(null);
        
        // -- create the basic connection factory
        ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(url, username, password);

        // -- create the pooledconnection factory
        PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(connectionFactory,
                connectionPool, null, null, false, true);

        System.out.println(poolableConnectionFactory.makeObject());
        // -- create the datasource
        dataSource = new PoolingDataSource(connectionPool);
    }

    public void disconnect() throws Exception {
        dataSource = null;
    }
    
// -- Getters -----------------------------------------------------------------
    
    public DataSource getDataSource() {
    	return dataSource;
    }
    
// -- Setters -----------------------------------------------------------------
    
    public void setDriverClass(String driverClass) {
		this.driverClass = driverClass;
	}
    
    public void setUrl(String url) {
		this.url = url;
		
		int index = this.url.lastIndexOf("/");
		
		if ((index < 0) || (index > this.url.length() - 1)) {
			catalog = "database";
		} else {
			catalog = this.url.substring(index + 1);
		}
	}
    
    public void setUsername(String username) {
		this.username = username;
	}
    
    public void setPassword(String password) {
		this.password = password;
	}
    
// -- Getters -----------------------------------------------------------------
    
    public String getDriverClass() {
		return driverClass;
	}
    
    public String getPassword() {
		return password;
	}
    
    public String getUrl() {
		return url;
	}
    
    public String getUsername() {
		return username;
	}
    
    public String getCatalog() {
    	return catalog;
    }
}
