package net.sf.sql2java.common.parameters.handlers;

import java.util.HashMap;
import java.util.Map;

import net.sf.sql2java.conversion.KeywordGroup;
import net.sf.sql2java.utils.StringUtils;

public class SimpleParameterHandler implements ParameterHandler {
	private Map<String, String> replacements;
	
// -- Constructors ------------------------------------------------------------
	
	public SimpleParameterHandler() {
		this(new HashMap<String, String>());
	}
	
	public SimpleParameterHandler(String key, String value) {
		this();
		
		addReplacement(key, value);
	}
	
	public SimpleParameterHandler(Map<String, String> replacements) {
		this.replacements = replacements;
	}
	
// -- Replacement Manipulation ------------------------------------------------
	
	public void addReplacement(String key, String value) {
		replacements.put(key, value);
	}
	
// -- Implementation ----------------------------------------------------------

	@Override
	public boolean isElegible(String parameter) {
		return true;
	}

	@Override
	public String onParameter(String content, String parameter) {
		// -- create the result
		String result = content;
		
		// -- check if a convertor is needed
		KeywordGroup translator = StringUtils.getKeywordGroup(parameter);
		if (translator != null) parameter = parameter.substring((translator.getName() + ":").length());
		
		// -- separate the navigation expression from the modifiers
		String[] parts = parameter.split("@");
	
		// -- let's find our object
		String replacement = replacements.get(parts[0]);
		
		if (replacement == null) return content;
		
		replacement = (translator == null) ? replacement.toString() : translator.translate(replacement.toString());
		
		// -- apply the modifiers onto the replacement
		replacement = StringUtils.performModifiers(replacement, parts);
		
		// -- replace all occurrences of the parameter with the replacement
		result = result.replaceAll(
				StringUtils.parameterize(parameter), 
				replacement
		);
		
		// -- return the result
		return result;
	}
}
