package net.sf.sql2java.fetching.sql;

import java.util.Properties;

import net.sf.sql2java.common.beans.BreadCrumbs;
import net.sf.sql2java.common.beans.SourceFilter;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;

public class XMLSQLFilter implements SourceFilter {
	public enum Props { configFile, filterMode }
	private Properties properties;
	private XMLConfiguration configuration;
	
// -- Constructors ------------------------------------------------------------
	
	public XMLSQLFilter(Properties properties) throws ConfigurationException {
		this.properties = properties;
		this.configuration = new XMLConfiguration(properties.getProperty(Props.configFile.toString()));
	}
	
// -- Matching Methods --------------------------------------------------------

	@Override
	public boolean match(BreadCrumbs breadcrumbs) {
		boolean internalResult = isMarked(breadcrumbs.toString());
		
		return (isExclusiveFilter()) ? (! internalResult) : internalResult;
	}
	
	protected boolean isMarked(String path) {
		try {
			// -- get the result
			boolean result = configuration.getBoolean(path + "[@mark]", false);
			
			// -- return the result
			return result;
		} catch (Exception e) {
			return false;
		}
	}
	
	public boolean isInclusiveFilter() {
		return (properties.getProperty(Props.filterMode.toString()).equalsIgnoreCase("inclusive"));
	}
	
	public boolean isExclusiveFilter() {
		return (properties.getProperty(Props.filterMode.toString()).equalsIgnoreCase("exclusive"));
	}
}
