package net.sf.sql2java.generator;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.ContextParameterHandler;
import net.sf.sql2java.common.resources.Resource;
import net.sf.sql2java.conversion.Convertor;
import net.sf.sql2java.conversion.KeywordGroup;
import net.sf.sql2java.generator.template.Template;
import net.sf.sql2java.generator.template.pack.TemplatePack;
import net.sf.sql2java.generator.writer.CodeWriter;

public class GeneratorSupport {
	private StringParser parser;
	
	private CodeWriter writer;
	private Convertor convertor;
	private TemplatePack templatePack;

// -- Constructors ------------------------------------------------------------
	
	public GeneratorSupport(GeneratorCycle cycle, CodeWriter writer, Convertor convertor, TemplatePack templatePack) {
		this.writer = writer;
		this.convertor = convertor;
		this.templatePack = templatePack;
		this.parser = new StringParser(
				new ContextParameterHandler(cycle.getProperties())
		);
	}
	
// -- Template Resources ------------------------------------------------------
	
	public String getAbsoluteTemplateDirectory(String templateName) {
		return getAbsoluteTemplateDirectory(getTemplate(templateName));
	}
	
	public String getAbsoluteTemplateDirectory(Template template) {
		// -- create the resource object
		Resource result = new Resource();
		
		// -- append the parts
		result.append(convertor.translateTo(KeywordGroup.JAVA, writer.getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getTemplatePack().getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getPath()));
		
		// -- parse and return the result
		return parser.parse(result.getPath());
	}
	
	public String getAbsoluteTemplateFile(String templateName) {
		return getAbsoluteTemplateFile(getTemplate(templateName));
	}
	
	public String getAbsoluteTemplateFile(Template template) {
		// -- create the resource object
		Resource result = new Resource();
		
		// -- append the parts
		result.append(convertor.translateTo(KeywordGroup.JAVA, writer.getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getTemplatePack().getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getEntityName()));
		result.addRaw(convertor.translateTo(KeywordGroup.JAVA, template.getExtension()));
		
		// -- parse and return the result
		return parser.parse(result.getPath());
	}
	
	public String getRelativeTemplateDirectory(String templateName) {
		return getRelativeTemplateDirectory(getTemplate(templateName));
	}
	
	public String getRelativeTemplateDirectory(Template template) {
		// -- create the resource object
		Resource result = new Resource();
		
		// -- append the parts
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getPath()));
		
		// -- parse and return the result
		return parser.parse(result.getPath());
	}
	
	public String getRelativeTemplateFile(String templateName) {
		return getRelativeTemplateFile(getTemplate(templateName));
	}
	
	public String getRelativeTemplateFile(Template template) {
		// -- create the resource object
		Resource result = new Resource();
		
		// -- append the parts
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getPath()));
		result.append(convertor.translateTo(KeywordGroup.JAVA, template.getEntityName()));
		result.addRaw(convertor.translateTo(KeywordGroup.JAVA, template.getExtension()));
		
		// -- parse and return the result
		return parser.parse(result.getPath());
	}
	
// -- Helper Methods ----------------------------------------------------------
	
	public Template getTemplate(String templateName) {
		// -- get the template with the given name
		Template result = templatePack.getTemplate(templateName);
		
		// -- check if the template is valid
		if (result == null) 
			throw new RuntimeException(
					String.format("No template found with name %s", templateName)
			);
		
		// -- return the result
		return result;
	}
}
