/*
 * ClassUtils.java
 * 
 * Created on Jun 18, 2007, 4:38:15 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.utils;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.StringTokenizer;

/**
 *
 * @author dge2
 */
public abstract class ClassUtils {

    @SuppressWarnings("unchecked")
	public static Object executeConstructor(Class targetClass, Object ... attributes)
    throws InstantiationException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException, 
           NoSuchMethodException, 
           ClassNotFoundException 
    {
        // -- initialize the class array needed by the constructor descriptor
        Class[] classes = new Class[attributes.length];
        int i = 0;
        for (Object attr : attributes)
            classes[i++] = attr.getClass();
        
        // -- get the constructor
        Constructor constructor = targetClass.getConstructor(classes);
        
        // -- invoke the constructor giving us a new object
        return constructor.newInstance(attributes);
    }
    
    @SuppressWarnings("unchecked")
	public static Object executeConstructor(String classname, Object ... attributes) 
    throws InstantiationException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException, 
           NoSuchMethodException, 
           ClassNotFoundException 
    {
        // -- get the class
        Class targetClass = Class.forName(classname);

        // -- execute the constructor
        return executeConstructor(targetClass, attributes);
    }
    
    public static Object executeGetter(String fieldname, Object target) 
    throws NoSuchMethodException, 
           SecurityException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException 
    {
        // -- create the result
        Object result = null;
        
        // -- create the fieldname
        String fn = "get" + fieldname.substring(0, 1).toUpperCase() + fieldname.substring(1);
        
        // -- execute the method to get the result
        result = executeMethod(fn, target);
        
        // -- return the result
        return result;
    }
    
    public static Object executeSetter(String fieldname, Object target, Object value) 
    throws NoSuchMethodException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException 
    {
        // -- create the settername
        String sn = "set" + fieldname.substring(0, 1).toUpperCase() + fieldname.substring(1);
        
        // -- invoke the method
        executeMethod(sn, target, value);
        
        // -- return the target object
        return target;
    }
    
    public static Object executeAdder(String fieldname, Object target, Object value) 
    throws NoSuchMethodException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException 
    {
        // -- create the settername
        String sn = "add" + fieldname.substring(0, 1).toUpperCase() + fieldname.substring(1);
        
        // -- invoke the method
        executeMethod(sn, target, value);
        
        // -- return the target object
        return target;
    }
    
    public static Object executeLookup(String fieldname, Object target, Object value) 
    throws NoSuchMethodException, 
           IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException 
    {
        // -- create the result
        Object result = null;
        
        // -- create the settername
        String sn = "lookup" + fieldname.substring(0, 1).toUpperCase() + fieldname.substring(1);
        
        // -- invoke the method
        result = executeMethod(sn, target, value);
        
        // -- return the target object
        return result;
    }
    
    @SuppressWarnings("unchecked")
	private static Object executeMethod(String methodname, Object targetObject, Object ... attributes) 
    throws IllegalAccessException, 
           IllegalArgumentException, 
           InvocationTargetException, 
           NoSuchMethodException, 
           SecurityException 
    {
        // -- create the result object
        Object result = null;
        
        // -- initialize the class array needed by the method descriptor
        Class[] classes = new Class[attributes.length];
        int i = 0;
        for (Object attr : attributes)
            classes[i++] = attr.getClass();
        
        // -- get the method
        Method method = targetObject.getClass().getMethod(methodname, classes);
        
        // -- invoke the method
        result = method.invoke(targetObject, attributes);
        
        // -- return the result object
        return result;
    }

    public static Object getValue(String expression, Object target)
    throws NoSuchMethodException, 
    	   SecurityException, 
    	   IllegalAccessException, 
    	   IllegalArgumentException, 
    	   InvocationTargetException 
    {
    	Object result = target;
    	StringTokenizer tokenizer = new StringTokenizer(expression, "#");
    	
    	while (tokenizer.hasMoreTokens()) {
    		result = executeGetter(tokenizer.nextToken(), result);
    	}
    	
    	if (target.equals(result)) return null;
    	else return result;
    }
}
